; ================================================================================================
; Mini-DMX-Empfangsroutine fr 96-768 Kanle (Festgelegt durch Header-Byte)
; Alle empfangenen Kanle werden gespeichert, Startadresse 1
; Besttigung wird in _ISR_Byte_Transmitted.inc im Interrupt versendet
;
; by Pesi - V 1.0 - 25.12.2010
; ================================================================================================
; benutzte Register      : temp0, temp1, ZL, ZH, werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : MDMX_State, MDMX_Framecount_L, MDMX_Framecount_H, MDMX_L, MDMH_H
;
; Definitionen (zum Kopieren):
;
;.def MDMX_State			= R21			; Receiver-Status, Register muss >15 sein
;.def MDMX_Framecount_L	= R22			; Zhler fr empfangene Bytes
;.def MDMX_Framecount_H	= R23			; Zhler fr empfangene Bytes
;.def MDMX_L				= R2			; Low-Byte Adress-Pointer
;.def MDMX_H				= R3			; High-Byte Adress-Pointer
; 
;.equ MDMX_Buffer	= 0x60				; RAM-Bereich fr empfangene Daten
; ------------------------------------------------------------------------------------------------

_ISR_Byte_Received:

		push	temp1							; Register sichern
		in		temp1, SREG
		push	temp1
		push	temp0
		push	ZL
		push	ZH

		in		temp0,UCSRA						; Status und Daten holen
		in		temp1,UDR						; Daten in temp1

		sbrc	temp0,FE						; Frame-Error: Auf Anfang
		rjmp	_ISR_MDMX_Clear

		cpi		MDMX_State, 0					; 0 = warten auf Blockstart 0x5A
		breq	_MDMX_Blockstart

		cpi		MDMX_State, 1					; 1 = warten auf Framesize-Byte
		breq	_MDMX_Header

		cpi		MDMX_State, 2					; 2 = Daten speichern
		breq	_ISR_MDMX_Store_Byte					

		cpi		MDMX_State, 3					; 3 = warten auf Blockende 0xA5
		breq	_MDMX_Blockende

		rjmp	_ISR_MDMX_Clear					; Zustand undefiniert, Ende

; ------------------------------------------------------------------------------------------------
; 0 = Warten auf Blockstart
; ------------------------------------------------------------------------------------------------

_MDMX_Blockstart:

		cpi		temp1, 0x5A						; Blockstart-Byte empfangen...?
		brne	_ISR_MDMX_Clear					; nein, Fehler, auf Blockende warten
		ldi		MDMX_State, 1					; ja, nchster Aufruf: auf Header warten

		rjmp	_ISR_MDMX_Ende					; und raus

; ------------------------------------------------------------------------------------------------
; 1 = Header auswerten - 0xA0 = 96 Kanle, 0xA1 = 256 Kan., 0xA2 = 512 Kan., 0xB0 = 768 Kan.
; ------------------------------------------------------------------------------------------------

_MDMX_Header:

		cpi		temp1, 0xA0						; mDMX-Header-Byte 96 Kanle empfangen...?
		brne	_MDMX_Header_256				; nein, auf 256 Kanle-Modus prfen
		
		ldi		MDMX_Framecount_L, low(96)		; ja, Framecounter auf 96 setzen
		ldi		MDMX_Framecount_H, high(96)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_256:

		cpi		temp1, 0xA1						; mDMX-Header-Byte 256 Kanle empfangen...?
		brne	_MDMX_Header_512				; nein, auf 512 Kanle-Modus prfen
		
		ldi		MDMX_Framecount_L, low(256)		; ja, Framecounter auf 256 setzen
		ldi		MDMX_Framecount_H, high(256)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_512:

		cpi		temp1, 0xA2						; mDMX-Header-Byte 512 Kanle empfangen...?
		brne	_MDMX_Header_768				; nein, auf 768 Kanle-Modus prfen
		
		ldi		MDMX_Framecount_L, low(512)		; ja, Framecounter auf 512 setzen
		ldi		MDMX_Framecount_H, high(512)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_768:

		cpi		temp1, 0xB0						; mDMX-Header-Byte 768 Kanle empfangen...?
		brne	_ISR_MDMX_Clear					; nein, Fehler
		
		ldi		MDMX_Framecount_L, low(768)		; ja, Framecounter auf 768 setzen
		ldi		MDMX_Framecount_H, high(768)

_MDMX_Header_Finish:

		ldi		MDMX_State, 2					; nchster Aufruf: Daten speichern

		ldi		ZH, high(MDMX_Buffer)			; Adresspointer auf Empfangspuffer setzen
		ldi		ZL, low(MDMX_Buffer)

		mov		MDMX_L, ZL						; Adresspointer speichern
		mov		MDMX_H, ZH

		rjmp	_ISR_MDMX_Ende					; und raus
	
; ------------------------------------------------------------------------------------------------
; 2 = Pixeldaten speichern
; ------------------------------------------------------------------------------------------------

_ISR_MDMX_Store_Byte:

		mov		ZL, MDMX_L						; Pointer holen
		mov		ZH, MDMX_H

		st		Z+, temp1						; empfangenes Byte speichern

		mov		MDMX_L, ZL						; Adresspointer sichern
		mov		MDMX_H, ZH

		subi	MDMX_Framecount_L, 1			; Framecounter runterzhlen
		sbci	MDMX_Framecount_H, 0			; in 16 Bit

		brne	_ISR_MDMX_Ende					; Frame noch nicht durch, Ende

		ldi		MDMX_State, 3					; Frame durch, warten auf Blockende

		rjmp	_ISR_MDMX_Ende					; und raus


; ------------------------------------------------------------------------------------------------
; 3 = Warten auf Blockende
; ------------------------------------------------------------------------------------------------

_MDMX_Blockende:

		cpi		temp1, 0xA5						; Blockende-Byte empfangen...?
		brne	_ISR_MDMX_Clear					; nein, Fehler, keine Besttigung, Receiver Reset

		ldi		temp1, 1						; Transmitter-Status: Ack Senden
		mov		tx_State, temp1
		ldi		temp1, 0x5A						; Blockstart-Byte Besttigung
		out		UDR, temp1						; Byte ausgeben (restliche Bytes in ISR)

		dec		Blink							; Status-LED toggeln
		sbrc	Blink, 0
		sbi		S_LED_Port, S_LED
		sbrs	Blink, 0
		cbi		S_LED_Port, S_LED

; ------------------------------------------------------------------------------------------------

_ISR_MDMX_Clear:

		clr		MDMX_State						; Receiver auf Anfang setzen
		cbi		UCSRA,FE						; FE-Flag lschen

_ISR_MDMX_Ende:

		pop		ZH								; Register wiederherstellen
		pop		ZL
		pop		temp0
		pop		temp1
		out		SREG, temp1
		pop		temp1

		reti									; Ende Routine

; ================================================================================================
